<?php

/**
 * Klasse zum generieren einer Captcha
 */
class CaptchaX
{

	var $imgWidth = 120;

	var $imgHeight = 40;

	var $textPadding = array(5, 6, 8, 11);

	var $noise = 50;

	var $dust_vs_scratches = 90;

	var $num_chars = 4;

	var $letters = 'a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, q, r, s, t, u, v, w, x, y, z, A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P, Q, R, S, T, U, V, W, X, Y, Z, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9';

	var $letter_precession = 15;

	var $use_local_fonts = true;

	var $font_sizes = array(22, 24, 26);

	var $fonts = array('basicdots.ttf', 'concielian.ttf', 'flytnspr.ttf');

	var $bg_color1 = array(255, 255, 255);

	var $bg_color2 = array(180, 239, 171);

	var $fg_color1 = array(225, 8, 8);

	var $fg_color2 = array(225, 8, 166);

	var $fg_color3 = array(60, 174, 44);

	var $fg_color4 = array(39, 79, 209);

	var $image;

	function __construct()
	{
		define('COLOR_VALUE_R', 0);
		define('COLOR_VALUE_G', 1);
		define('COLOR_VALUE_B', 2);
		define('CASE_SENSITIVE', false);
	}

	/**
	 * Zeigt das bild On-the-Fly an
	 *
	 * @param integer $code CaptchaId
	 */
	function showImage($code)
	{
		$this->image = $this->createImageBase();
		$this->addDustAndScratches($this->bg_color2);
		$this->printCaptchaString($code);
		$this->addDustAndScratches($this->bg_color_1);
		header("Content-type: image/jpeg");
		imagejpeg($this->image);
		imagedestroy($this->image);
	}

	/**
	 * Erstellt das Bildobjekt
	 *
	 * @return ressource
	 */
	function createImageBase()
	{
		$image = imagecreate($this->imgWidth, $this->imgHeight);
		imagecolorallocate($image, $this->bg_color1[COLOR_VALUE_R], $this->bg_color1[COLOR_VALUE_G], $this->bg_color1[COLOR_VALUE_B]);
		return $image;
	}

	/**
	 * fuegt zum Bild Kratzer und Schmutz
	 *
	 * @param string $color Farbe
	 */
	function addDustAndScratches($color)
	{
		$max_x = $this->imgWidth - 1;
		$max_y = $this->imgHeight - 1;
		$color = imagecolorallocate($this->image, $color[COLOR_VALUE_R], $color[COLOR_VALUE_G], $color[COLOR_VALUE_B]);
		for($i = 0; $i < $this->noise; ++ $i)
		{
			if (rand(1, 100) > $this->dust_vs_scratches)
			{
				imageline($this->image, rand(0, $max_x), rand(0, $max_y), rand(0, $max_x), rand(0, $max_y), $color);
			}
			else
			{
				imagesetpixel($this->image, rand(0, $max_x), rand(0, $max_y), $color);
			}
		}
	}

	/**
	 * Generiert das Captcha-Bild
	 *
	 * @param integer $code CaptchaId
	 */
	function printCaptchaString($code)
	{
		list($padding_top, $padding_right, $padding_bottom, $padding_left) = $this->textPadding;
		$box_width = ($this->imgWidth - ($padding_left + $padding_right)) / $this->num_chars;
		$box_height = $this->imgHeight - ($padding_top + $padding_bottom);
		$font_size = $this->font_sizes;
		$font_size_count = (count($font_size) - 1);
		$fonts = $this->fonts;
		$fonts_count = (count($fonts) - 1);
		$fg_colors_count = 4;
		$fg_colors = array($this->fg_color1, $this->fg_color2, $this->fg_color3, $this->fg_color4);
		for($i = 0; $i < $this->num_chars; ++ $i)
		{
			$size_index = rand(0, $font_size_count);
			$size = $font_size[$size_index];
			$angle = ((rand(0, ($this->letter_precession * 2)) - $this->letter_precession) + 360) % 360;
			$x = $padding_left + ($box_width * $i);
			$y = $padding_top + $size + (($box_height - $size) / 2);
			$color_index = (rand(0, $fg_colors_count));
			$color = $fg_colors[$color_index];
			$color = imagecolorallocate($this->image, $color[0], $color[1], $color[2]);
			$font_index = rand(0, $fonts_count);
			$font = $fonts[$font_index];
			imagettftext($this->image, $size, $angle, $x, $y, $color, dirname(__FILE__) . '/fonts/' . $font, $_SESSION['plugins']['CaptchaX'][$code][$i]);
		}
	}

	/**
	 * generiert die CaptchaId
	 *
	 * @return integer
	 */
	function getCaptchaId()
	{
		$letters = explode(', ', $this->letters);
		$letters_max = count($letters) - 1;
		$randomString = array();
		for($i = 0; $i < $this->num_chars; ++ $i)
		{
			$letter_index = rand(0, $letters_max);
			$randomString[] = $letters[$letter_index];
		}
		if (! isset($_SESSION['plugins']['CaptchaX']))
		{
			$_SESSION['plugins']['CaptchaX'] = array();
		}
		$microtime = str_replace('.', '', microtime(true));
		$_SESSION['plugins']['CaptchaX'][$microtime] = $randomString;
		return $microtime;
	}

	/**
	 * Ueberpruefen ob Code korrekt eingeben ist
	 *
	 * @param string $user_string Code aus Formular
	 * @param string $code CaptchaId
	 * @return boolean
	 */
	function validateCaptchaString($user_string, $code)
	{
		if (! isset($_SESSION['plugins']['CaptchaX'][$code]))
		{
			return false;
		}
		$old_string = $_SESSION['plugins']['CaptchaX'][$code];
		unset($_SESSION['plugins']['CaptchaX'][$code]);
		return ((strtolower(implode($old_string))) == (strtolower($user_string)));
	}

}
