<?php

/*
Plugin Name: simpleContact
Plugin URI: http://www.goizio.com/simplecontact/
Description: Kontaktformular f&uuml;r Wordpress mit einer einfachen Captcha-Abfrage
Version: v1.01
Author: Mathias 'United20' Schmidt
Author URI: http://www.goizio.com/
*/

/**
 * laden der Captcha- und RegEx-Klasse
 */
require_once (dirname(__FILE__) . '/CaptchaX/CaptchaX.class.php');
require_once (dirname(__FILE__) . '/RegEx/RegEx.class.php');

/**
 * Initialisieren der Klasse
 */
$sc = new simpleContact();
add_action('init', array($sc, 'init'));
add_action('admin_menu', array($sc, 'adminMenu'));

/**
 * Filter fuer Funktionen
 */
add_filter('the_content', array($sc, 'replacePlaceholder'));

/**
 * Einstellungen
 */
add_option('simplecontact_captcha', 'on', 'Soll ein Captcha fuer verwendet werden, um SPAM ab zu wehren.', 'yes');
add_option('simplecontact_upload', 'on', 'Ist es moeglich Bilder mit dem Kontaktformular hochzuladen.', 'yes');
add_option('simplecontact_upload_formates', 'a:3:{s:3:"jpg";a:3:{s:4:"name";s:4:"JPEG";s:4:"mime";a:1:{i:0;s:10:"image/jpeg";}s:10:"extensions";a:3:{i:0;s:3:"jpg";i:1;s:4:"jpeg";i:2;s:3:"jpe";}}s:3:"png";a:3:{s:4:"name";s:3:"PNG";s:4:"mime";a:1:{i:0;s:9:"image/png";}s:10:"extensions";a:1:{i:0;s:3:"png";}}s:3:"gif";a:3:{s:4:"name";s:3:"GIF";s:4:"mime";a:1:{i:0;s:9:"image/gif";}s:10:"extensions";a:1:{i:0;s:3:"gif";}}}', 'Die Datentypen, welche fuer den Bilderupload zugelassen sind.', 'yes');
add_option('simplecontact_max_upload_size', '1048576', 'Maximale Upload Groesse fuer Dateien.', 'yes');
add_option('simplecontact_message_min_lenght', '3', 'Mindestlaenge fuer eine Nachricht.', 'yes');

/**
 * Thickbox hinzufuegen
 */
add_thickbox();

/**
 * Debugfunktionen
 */
function vd($var)
{
	echo '<pre>';
	var_dump($var);
	echo '</pre>';
}

class simpleContact
{

	/**
	 * Platzhalter fuer das Kontaktformular
	 *
	 * @var string
	 */
	var $placeholder = '<!-- simple-contact -->';

	/**
	 * der aktuelle Beitrag
	 *
	 * @var object
	 */
	var $post = null;

	/**
	 * Die Daten aus dem Kontaktformular
	 *
	 * @var array
	 */
	var $form = null;

	/**
	 * die Eingabefehler
	 *
	 * @var array
	 */
	var $errors = array();

	/**
	 * wurde das Formular ohne Fehler abgeschickt und konnten die Daten in der Datenbank gespeichert werden
	 *
	 * @var boolean
	 */
	var $send = false;

	/**
	 * das Datenbankobjekt von Wordpress
	 *
	 * @var object
	 */
	var $db = null;

	/**
	 * Request Daten
	 *
	 * @var array
	 */
	var $request = array();

	/**
	 * Config
	 *
	 * @var array
	 */
	var $config = null;

	/**
	 * Ordner im Upload-Verzeichnis
	 *
	 * @var string
	 */
	var $folder = '/sc-attachment/';

	/**
	 * Formate fuer den Dateiupload
	 *
	 * @var array
	 */
	var $formates = array('jpg' => array('name' => 'JPEG', 'mime' => array('image/jpeg'), 'extensions' => array('jpg', 'jpeg', 'jpe')), 'gif' => array('name' => 'GIF', 'mime' => array('image/gif'), 'extensions' => array('gif')), 'png' => array('name' => 'PNG', 'mime' => array('image/png'), 'extensions' => array('png')), 'doc' => array('name' => 'Worddokument', 'mime' => array('application/msword'), 'extensions' => array('doc', 'docx', 'dot')), 'pdf' => array('name' => 'PDF-Dokument', 'mime' => array('application/pdf'), 'extensions' => array('pdf')), 'txt' => array('name' => 'TXT-Datei', 'mime' => array('text/plain'), 'extensions' => array('txt')));

	/**
	 * aktuelle Kontaktnachricht
	 *
	 * @var $array
	 */
	var $contact = null;

	/**
	 * Konstruktor der Klasse
	 * holt die ueber das Formular verschickten Daten
	 */
	function simpleContact()
	{
		session_start();
		$this->loadSettings();
		$this->initWpDB();
		$this->firstRun();
		$this->requestData();
		if (isset($_POST['simple_contact']) && is_array($_POST['simple_contact']) && count($_POST['simple_contact']))
		{
			$this->form = $_POST['simple_contact'];
			if ($this->validate())
			{
				$this->send = $this->saveForm() !== false;
			}
		}
	}

	/**
	 * laedt die Einstellungen fuer das Plugin
	 */
	function loadSettings()
	{
		if (empty($this->config))
		{
			$this->config = array(
				'captcha' => $this->installedGDLibary() && get_option('simplecontact_captcha', true) == 'on',
				'upload' => get_option('simplecontact_upload', true) == 'on',
				'upload_formates' => unserialize(get_option('simplecontact_upload_formates', true)),
				'upload_size' => get_option('simplecontact_max_upload_size', true),
				'upload_path' => get_option('upload_path', true),
				'message_min_length' => get_option('simplecontact_message_min_lenght', true)
			);
			$this->config['upload_path'] = $this->getUploadPath();
			$this->config['upload_web_path'] = $this->getUploadWebPath();
		}
	}

	/**
	 * Testen, ob die GD Library installiert ist
	 *
	 * @return boolean
	 */
	function installedGDLibary()
	{
		return @extension_loaded('gd');
	}

	/**
	 * Zusammenbauen des gesamten Uploadpfades und erstellen des Uploadverzeichnisses wenn es nicht existiert
	 *
	 * @return string
	 */
	function getUploadPath()
	{
		$path = $this->config['upload_path'] . '/' . trim($this->folder, '/') . '/';
		if (! file_exists($path))
		{
			@mkdir($path, '0777');
			@chmod($path, '0777');
		}
		return $path;
	}

	/**
	 * Zusammenbauen des gesamten Webpaths zum Uploadverzeichnis
	 *
	 * @return string
	 */
	function getUploadWebPath()
	{
		$upload = wp_upload_dir();
		return trim($upload['baseurl'], '/') . '/' . trim($this->folder, '/') . '/';
	}

	/**
	 * wenn das Script das erste Mal aufgerufen wird, dann werden die Tabellen fuer das Plugin erstellt
	 */
	function firstRun()
	{
		$tables = $this->fetchTables();
		if (! is_int($this->query('SELECT * FROM #simple_contact_subjects')))
		{
			$this->query('
				CREATE TABLE #simple_contact_subjects (
					subject_id int(10) unsigned not null auto_increment,
					subject_name varchar(100) not null,
					subject_order int(10) unsigned not null default "0",
					subject_active tinyint(1) unsigned not null default "1",
					PRIMARY KEY(subject_id)
				);
			');
			$this->query('INSERT INTO #simple_contact_subjects (subject_id, subject_name, subject_order, subject_active) VALUES (NULL, "Allgemeine Frage", 1, 1);');
			$this->query('INSERT INTO #simple_contact_subjects (subject_id, subject_name, subject_order, subject_active) VALUES (NULL, "Verbesserungsvorschlag", 2, 1);');
			$this->query('INSERT INTO #simple_contact_subjects (subject_id, subject_name, subject_order, subject_active) VALUES (NULL, "Fehler melden", 3, 1);');
			$this->query('INSERT INTO #simple_contact_subjects (subject_id, subject_name, subject_order, subject_active) VALUES (NULL, "Werbeanfrage", 4, 1);');
		}
		if (! is_int($this->query('SELECT * FROM #simple_contact_messages')))
		{
			$this->query('
				CREATE TABLE #simple_contact_messages (
					message_id int(10) unsigned not null auto_increment,
					subject_id int(10) unsigned not null,
					message_username varchar(255) not null,
					message_email varchar(255) not null,
					message_date int(10) unsigned not null,
					message_ip varchar(40) not null default "127.0.0.1",
					message_message text not null,
					message_attachment varchar(255) not null,
					message_read tinyint(1) unsigned not null default 0,
					PRIMARY KEY (message_id)
				);
			');
		}
	}

	/**
	 * initialisieren des Datenbankobjektes
	 */
	function initWpDB()
	{
		global $wpdb;
		if (! defined('DB_PREFIX'))
		{
			define('DB_PREFIX', $wpdb->prefix);
		}
		$this->db = $wpdb;
	}

	/**
	 * holen der Request-Daten ueber die URL
	 */
	function requestData()
	{
		if (isset($_GET['username']))
		{
			$this->request['username'] = htmlspecialchars(stripcslashes($_GET['username']));
		}
		if (isset($_GET['email']) && preg_match(RegEx::TmplEMail(), $_GET['email']))
		{
			$this->request['email'] = $_GET['email'];
		}
		if (isset($_GET['subject']) && preg_match('/^[0-9]{0,10}$/', $_GET['subject']))
		{
			$this->request['subject'] = $_GET['subject'];
		}
	}

	/**
	 * fuehrt einen MySQL-Query aus
	 *
	 * @param string $query
	 * @return ressource
	 */
	function query($query)
	{
		return $this->db->query($this->replacePrefix($query));
	}

	/**
	 * Loeschen eines oder mehrerer Datensaetze aus der Datenbank
	 *
	 * @param string $table
	 * @param string $where
	 * @return integer
	 */
	function delete($table, $where)
	{
		return $this->query('DELETE FROM ' . $table . ' WHERE ' . $where);
	}

	/**
	 * gibt die Resultate eines Queries zurueck
	 *
	 * @param string $query
	 * @return array
	 */
	function fetchResults($query)
	{
		return $this->db->get_results($this->replacePrefix($query));
	}

	/**
	 * gibt alle Tabellen von Wordpress zurueck
	 *
	 * @return array
	 */
	function fetchTables()
	{
		return $this->db->tables;
	}

	/**
	 * gibt ein assoziatives Array zurueck, welches zusammengehoerige Paare zurueck gibt
	 *
	 * @param string $query
	 * @return array
	 */
	function fetchPairs($query)
	{
		$return = array();
		$data = $this->fetchResults($query);
		foreach ($data as $object)
		{
			$return[$object->subject_id] = $object->subject_name;
		}
		return $return;
	}

	/**
	 * ersetzen des Platzhalters # in einem Query (wie ZendDB)
	 *
	 * @param string $query
	 * @return string
	 */
	function replacePrefix($query)
	{
		return str_replace('#', DB_PREFIX, $query);
	}

	/**
	 * ersetzen des Platzhalters im Content der Seite
	 *
	 * @param string $content
	 * @return string
	 */
	function replacePlaceholder($content)
	{
		if (substr_count($content, $this->placeholder))
		{
			$content = str_replace($this->placeholder, $this->showFormular(), $content);
		}
		return $content;
	}

	/**
	 * den aktuellen Post und seine Daten holen
	 *
	 * @param boolean $reload
	 * @return array
	 */
	function getCurPost($reload = false)
	{
		global $wp_query;
		if (empty($this->post) || $reload)
		{
			$this->post = $wp_query->get_queried_object();
			if (empty($this->post))
			{
				global $post;
				$this->post = $post;
			}
		}
		return $this->post;
	}

	/**
	 * anzeigen des Kontaktformulars bzw. der Danke-Seite
	 *
	 * @return string
	 */
	function showFormular()
	{
		if ($this->send == true)
		{
			require_once (dirname(__FILE__) . '/templates/done.tpl.php');
			return $tpl;
		}
		else
		{
			$subjects = $this->getSubjects();

			// Captcha laden
			$captcha = new CaptchaX();
			$captchaId = $captcha->getCaptchaId();
			$request = $this->request;
			$formats = $this->getFormats();
			require_once (dirname(__FILE__) . '/templates/formular.tpl.php');
			return $tpl;
		}
	}

	/**
	 * holen der Betreffs fuer das Kontaktformular
	 *
	 * @return array
	 */
	function getSubjects()
	{
		return $this->fetchPairs('SELECT subject_id, subject_name FROM #simple_contact_subjects WHERE subject_active=1 ORDER BY subject_order ASC, subject_name ASC');
	}

	/**
	 * Captcha-Validierung
	 *
	 * @param integer $captchaId ID fuer das Captcha
	 * @param string $code eingebener Code
	 * @return boolean
	 */
	function captcha($captchaId, $code)
	{
		$captcha = new CaptchaX();
		return $captcha->validateCaptchaString($code, $captchaId);
	}

	/**
	 * Validieren der Eingaben und gibt zurueck, ob die eingabe valide ist
	 *
	 * @return boolean
	 */
	function validate()
	{
		if (strlen(trim($this->form['username'])) <= 0 || strlen(trim($this->form['username'])) > 100 || ! preg_match(RegEx::TmplASCII(), $this->form['username']))
		{
			$this->errors[] = 'error username';
		}
		if (! preg_match(RegEx::TmplEMail(), $this->form['email']))
		{
			$this->errors[] = 'error email';
		}
		if (! $this->form['subject'])
		{
			$this->errors[] = 'error subject';
		}
		if (strlen(trim($this->form['message'])) < $this->config['message_min_length'])
		{
			$this->errors[] = 'error message';
		}
		if ($this->config['upload'] && isset($_FILES['simple_contact']) && strlen(trim($_FILES['simple_contact']['name']['file'])) && ! $this->validateFiletype($_FILES['simple_contact']))
		{
			$this->errors[] = 'error filetype';
		}
		if (! $this->checkFileSize($_FILES['simple_contact']['size']['file']))
		{
			$this->errors[] = 'error filesize';
		}
		if ($this->config['captcha'] && ! $this->captcha($this->form['captcha_id'], $this->form['code']))
		{
			$this->errors[] = 'error captcha';
		}
		return ! count($this->errors);
	}

	/**
	 * prueft ob ein Fehler vorhanden ist oder nicht
	 *
	 * @param string $type
	 * @return boolean
	 */
	function hasError($type)
	{
		return in_array($type, $this->errors);
	}

	/**
	 * holt die Formate fuer den Dateiupload
	 *
	 * @return string
	 */
	function getFormats()
	{
		if (count($this->config['upload_formates']))
		{
			$formats = array();
			foreach ($this->config['upload_formates'] as $value)
			{
				$formats[] = $value['name'];
			}
			return implode(', ', $formats);
		}
		return false;
	}

	/**
	 * Validierung des Datentyps des Dateianhangs
	 *
	 * @param array $file
	 * @return boolean
	 */
	function validateFiletype($file)
	{
		return $this->checkFileType($file['type']['file']) && $this->checkExtension($file['name']['file']);
	}

	/**
	 * Uberpruefung des MIME-Types
	 *
	 * @param string $type
	 * @return boolean
	 */
	function checkFileType($type)
	{
		foreach ($this->config['upload_formates'] as $format)
		{
			if (in_array($type, $format['mime']))
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Ueberpruefen der maximalen Upload-Groesse
	 *
	 * @param string $size
	 * @return boolean
	 */
	function checkFileSize($size)
	{
		return $this->config['upload_size'] && ($size <= $this->config['upload_size']);
	}

	/**
	 * Pruefen der Dateiendung
	 *
	 * @param string $file
	 * @return boolean
	 */
	function checkExtension($file)
	{
		$ext = $this->getExtension($file);
		foreach ($this->config['upload_formates'] as $format)
		{
			if ($ext && in_array($ext, $format['extensions']))
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * holt die Dateiendung eines Dateinamens
	 *
	 * @param string $file
	 * @return string
	 */
	function getExtension($file)
	{
		$pos = strrpos($file, ".");
		if ($pos !== false)
		{
			return strtolower(substr($file, $pos + 1));
		}
		return false;
	}

	/**
	 * speichern der Daten des Kontaktformulars
	 *
	 * @return integer
	 */
	function saveForm()
	{
		$upload = '';
		if ($this->config['upload'] && isset($_FILES['simple_contact']) && strlen(trim($_FILES['simple_contact']['name']['file'])))
		{
			$upload = $this->uploadFile($_FILES['simple_contact']);
		}
		$insert = array('subject_id' => $this->form['subject'], 'message_username' => $this->form['username'], 'message_email' => $this->form['email'], 'message_date' => time(), 'message_ip' => getenv('REMOTE_ADDR'), 'message_message' => $this->form['message'], 'message_attachment' => $upload);
		return $this->insert('#simple_contact_messages', $insert, array('ignore' => true));
	}

	/**
	 * Fuegt einen Datensatz in eine Datenbanktabelle ein
	 *
	 * @param string $table
	 * @param array $fields
	 * @param array $options
	 * @return integer
	 */
	function insert($table, $fields, $options = array())
	{
		$query = 'INSERT ';
		if (isset($options['ignore']) && $options['ignore'] == true)
		{
			$query .= 'IGNORE ';
		}
		$query .= 'INTO ' . $table . ' SET ';
		$queryFields = array();
		foreach ($fields as $key => $value)
		{
			if (preg_match('/^[0-9]+$/', $key))
				$queryFields[] = addslashes($value);
			else
				$queryFields[] = $key . '="' . addslashes($value) . '"';
		}
		$this->query($query . implode(', ', $queryFields));
		return mysql_insert_id();
	}

	/**
	 * laedt die angehaengte Datei in ein spezifiziertes Verzeichnis und gibt den Dateinamen zurueck
	 *
	 * @param array $file
	 * @return string
	 */
	function uploadFile($file)
	{
		$filename = time() . '-' . $this->convertToUrlName($file['name']['file']);
		$upload = $this->config['upload_path'] . $filename;
		if (move_uploaded_file($file['tmp_name']['file'], $upload))
		{
			return $filename;
		}
		return '';
	}

	/**
	 * aendert einen String in einen URL-Namen
	 *
	 * @param string $input
	 * @return string
	 */
	function convertToUrlName($input)
	{
		$output = strtolower(html_entity_decode($input));
		$transTable = array('�' => 'ae', '�' => 'oe', '�' => 'ue', '�' => 'Ae', '�' => 'Oe', '�' => 'Ue', '�' => 'ss');
		$output = strtr($output, $transTable);
		$output = preg_replace('/[^A-Za-z0-9-\.]/', '-', $output);
		$output = preg_replace('/-+/', '-', $output);
		$output = trim($output, '-');
		return $output;
	}

	/**
	 * Initialisiert die Multisprachfaehigkeit
	 */
	function init()
	{
		load_plugin_textdomain('simpleContact', PLUGINDIR . '/' . dirname(plugin_basename(__FILE__)) . '/lang');
	}

	/**
	 * verknuepft die Einstellungsseite von izioSeo mit dem Adminmenu
	 */
	function adminMenu()
	{
		add_submenu_page('options-general.php', __('simpleContact', 'simpleContact'), __('simpleContact', 'simpleContact'), 10, __FILE__, array($this, 'options'));
	}

	/**
	 * Die Seite mit den Optionen
	 */
	function options()
	{
		// Formulare entgegen nehmen
		if (isset($_POST['simplecontact']))
		{
			$message = $this->saveSettings($_POST['simplecontact']);
		}
		elseif (isset($_POST['newsubject']))
		{
			$message = $this->addNewSubject($_POST['newsubject']);
			if ($message != 'newsubject')
			{
				$newsubject = $_POST['newsubject'];#
			}
		}
		elseif (isset($_POST['subject']))
		{
			$errors = $this->updateOrder($_POST['subject']);
			$message = count($errors) ? 'error order' : 'order';
		}
		else
		{
			// Status setzen
			if (isset($_GET['deactivate']) && preg_match('/^[0-9]+$/', $_GET['deactivate']))
			{
				$status = $this->setStatus($_GET['deactivate'], 'deactivate');
			}
			if (isset($_GET['activate']) && preg_match('/^[0-9]+$/', $_GET['activate']))
			{
				$status = $this->setStatus($_GET['activate'], 'activate');
			}
			if (isset($status))
			{
				$message = $status ? 'status' : null;
				$status = isset($_GET['activate']) ? $_GET['activate'] : $_GET['deactivate'];
			}
			// Betreff loeschen
			if (isset($_GET['delete']) && preg_match('/^[0-9]+$/', $_GET['delete']))
			{
				$delete = $this->deleteSubject($_GET['delete']);
				$message = $delete ? 'delete' : null;
				$delete = $_GET['delete'];
			}
			// Nachricht loeschen
			if (isset($_GET['m_delete']) && preg_match('/^[0-9]+$/', $_GET['m_delete']))
			{
				$contact = $this->getContact($_GET['m_delete']);
				$delete = $this->deleteMessage($_GET['m_delete']);
				$message = $delete ? 'message delete' : null;
				$name = $contact->message_username;
				$date = $contact->message_date;
			}
		}
		$data = array('simplecontact_captcha' => get_option('simplecontact_captcha', true), 'simplecontact_upload' => get_option('simplecontact_upload', true), 'simplecontact_upload_formates' => $this->unserializeFormates(get_option('simplecontact_upload_formates', true)), 'simplecontact_max_upload_size' => htmlspecialchars(stripcslashes(get_option('simplecontact_max_upload_size', true))) / 1024, 'simplecontact_message_min_lenght' => htmlspecialchars(stripcslashes(get_option('simplecontact_message_min_lenght', true))));
		$list = $this->getList();
		$gd = $this->installedGDLibary();
		require_once (dirname(__FILE__) . '/templates/option-panel.tpl.php');
	}

	/**
	 * speichert die Einstellungen fuer das Kontaktformular
	 *
	 * @param array $settings
	 * @return string
	 */
	function saveSettings($settings)
	{
		if (! empty($settings) && is_array($settings) && count($settings))
		{
			$settings['simplecontact_captcha'] = isset($settings['simplecontact_captcha']) && $settings['simplecontact_captcha'] == 'on' ? 'on' : 'off';
			$settings['simplecontact_upload'] = isset($settings['simplecontact_upload']) && $settings['simplecontact_upload'] == 'on' ? 'on' : 'off';
			$settings['simplecontact_max_upload_size'] = (int)($settings['simplecontact_max_upload_size'] * 1024);
			$settings['simplecontact_upload_formates'] = $this->serializeFormates($settings['simplecontact_upload_formates']);
			foreach ($settings as $key => $value)
			{
				if (strlen(trim($value)))
				{
					update_option($key, trim($value));
				}
			}
			if (function_exists('wp_cache_flush'))
			{
				wp_cache_flush();
			}
			return 'settings';
		}
		return false;
	}

	/**
	 * Setzen des Status fuer die Betreffs
	 *
	 * @param integer $id
	 * @param string $type
	 * @return integer
	 */
	function setStatus($id, $type)
	{
		$type = ($type == 'activate' ? 1 : 0);
		return $this->update('#simple_contact_subjects', array('subject_active' => $type), 'WHERE subject_id=' . addslashes($id));
	}

	/**
	 * Update eines Datensatzes (aehnlich ZendDB)
	 *
	 * @param string $table
	 * @param string/array $fields
	 * @param string $where
	 * @return integer
	 */
	function update($table, $fields, $where)
	{
		$query = 'UPDATE ' . $table . ' SET ';
		if (is_array($fields))
		{
			$values = array();
			foreach ($fields as $field => $value)
			{
				$values[] = $field . '="' . addslashes($value) . '"';
			}
			$query .= implode(',', $values);
		}
		else
		{
			$query .= $fields . ' ';
		}
		$query .= $where;
		return $this->query($query);
	}

	/**
	 * Fuegt einen neuen Betreff hinzu. Validiert diesen vorher und gibt ggfs. eine Fehlermeldung aus.
	 *
	 * @param array $subject
	 * @return string
	 */
	function addNewSubject($subject)
	{
		if (strlen(trim($subject['name'])) && preg_match('/^[0-9]+$/', $subject['order']))
		{
			$this->insert('#simple_contact_subjects', array('subject_name' => $subject['name'], 'subject_order' => $subject['order'], 'subject_active' => 1), array('ignore' => true));
			return 'newsubject';
		}
		return 'error newsubject';
	}

	/**
	 * unserialisiert die Dateitypen und bereitet Sie fuer das Formular des Adminpannels auf
	 *
	 * @param string $data
	 * @return array
	 */
	function unserializeFormates($data)
	{
		$formates = array();
		$data = unserialize($data);
		foreach ($data as $key => $value)
		{
			$formates[$key] = true;
		}
		return $formates;
	}

	/**
	 * serialisiert und ueberprueft die Dateitypen
	 *
	 * @param array $data
	 * @return string
	 */
	function serializeFormates($data)
	{
		$formates = array();
		foreach ($data as $key => $value)
		{
			$formates[$key] = $this->formates[$key];
		}
		return serialize($formates);
	}

	/**
	 * holt alle Betreffs und Kontaktanfragen
	 *
	 * @return array
	 */
	function getList()
	{
		$subject = null;
		$list = array();
		$res = $this->fetchResults('SELECT sub.subject_id AS id, sub.*, m.* FROM #simple_contact_subjects AS sub LEFT JOIN #simple_contact_messages AS m ON sub.subject_id=m.subject_id ORDER BY subject_order ASC, subject_name ASC');
		foreach ($res as $data)
		{
			if (empty($subject) || $data->subject_id != $subject)
			{
				$subject = $data->id;
				$list[$subject] = array('subject_id' => $subject, 'subject_name' => $data->subject_name, 'subject_order' => $data->subject_order, 'subject_active' => $data->subject_active, 'subject_messages' => array());
			}
			if (! empty($data->message_id))
			{
				$list[$subject]['subject_messages'][] = array('message_id' => $data->message_id, 'message_username' => $data->message_username, 'message_email' => $data->message_email, 'message_date' => $data->message_date, 'message_ip' => $data->message_ip, 'message_message' => $data->message_message, 'message_attachment' => $this->checkAttachment($data->message_attachment), 'message_read' => $data->message_read);
			}
		}
		return $list;
	}

	/**
	 * Ueberprueft ob der Anhang der Nachricht ueberhaupt vorhanden ist
	 *
	 * @param string $file
	 * @return string
	 */
	function checkAttachment($file)
	{
		if (! empty($file) && strlen(trim($file)))
		{
			if (file_exists($this->config['upload_path'] . $file))
			{
				return $this->config['upload_web_path'] . $file;
			}
		}
		return false;
	}

	/**
	 * Aktualisieren der Reihenfolge fuer die Betreffs
	 *
	 * @param array $subjects
	 * @return array
	 */
	function updateOrder($subject)
	{
		$errors = array();
		foreach ($subject['order'] as $id => $value)
		{
			if (preg_match('/^[0-9]+$/', $value))
			{
				$this->update('#simple_contact_subjects', array('subject_order' => addslashes($value)), 'WHERE subject_id=' . addslashes($id));
			}
			else
			{
				$errors[] = '#' . $id;
			}
		}
		return $errors;
	}

	/**
	 * Loeschen einer Kontaktnachricht
	 *
	 * @param integer $id
	 * @return string
	 */
	function deleteMessage($id)
	{
		$this->deleteAttachment($id);
		return $this->delete('#simple_contact_messages', 'message_id=' . addslashes($id));
	}

	/**
	 * Loeschen eines Betreffs
	 *
	 * @param integer $id
	 * @return string
	 */
	function deleteSubject($id)
	{
		$messages = $this->fetchResults('SELECT message_id FROM #simple_contact_messages WHERE subject_id=' . addslashes($id) .  ' AND message_attachment!=""');
		if ($messages)
		{
			foreach ($messages as $message)
			{
				$this->deleteAttachment($message['message_id']);
			}
		}
		$this->delete('#simple_contact_messages', 'subject_id=' . addslashes($id));
		return $this->delete('#simple_contact_subjects', 'subject_id=' . addslashes($id));
	}

	/**
	 * holen der Daten einer Kontaktnachricht
	 *
	 * @param integer $ids
	 * @param boolean $force
	 * @return object/array
	 */
	function getContact($ids, $force = false)
	{
		if (!is_array($ids))
			$ids = array($ids);
		if (empty($this->contact) || $force)
			$this->contact = $this->fetchResults('SELECT * FROM #simple_contact_messages WHERE message_id IN ("' . implode('","', $ids) . '")');

		if (count($this->contact) > 1)
			return $this->contact;
		elseif (count($this->contact) == 1)
			return $this->contact[0];

		return false;
	}

	/**
	 * Loeschen der Dateianhaenge
	 *
	 * @param integer $ids
	 */
	function deleteAttachment($ids)
	{
		if (!is_array($ids))
			$ids = array($ids);

		foreach ($ids as $id)
		{
			$contact = $this->getContact($id, true);
			$attachment = realpath($this->config['upload_path']) . '/' . $contact->message_attachment;
			if (!is_dir($attachment) && file_exists($attachment))
				@unlink($attachment);
		}
	}

}
